#include <Servo.h>
#include <math.h>

// This version uses the gamepad to control the robot motions.
// Left/right rotates arm about the base (theta)
// Up/down moves the claw vertically (z)
// Red/blue move the claw in and out radially (r)
// Red and blue simultaneously does a "wave" with the claw
// Yellow opens/closes the claw
// Green moves to "pickup" or "drop" locations for candy drop feature
// The claw orientation is maintained level to the ground.

// After each button release, the positions of all servos are written to the serial monitor.
// You can manually move the robot where you want it to go, then check the serial monitor to read the servo positions.

// A new mode was added where there is a choice of 4 candies to be picked up.
// Pressing a button causes the corresponding candy to be picked up.
// Pressing down will release the candy.

Servo rotate;  // create servo object to control a servo
Servo shoulder;  // create servo object to control a servo
Servo elbow;  // create servo object to control a servo
Servo wrist;  // create servo object to control a servo
Servo claw;  // create servo object to control a servo

// connections for the gamepad.  Only green button used here, but info is here for future use.
int hpin = A1;  // analog pin for left/right control.  left = 5V, right = 1.67V, center = 2.5V (approx.)
int vpin = A0;  // analog pin for up/down control.  up = 5V, down = 1.67V, center = 2.5V (approx.)
int redpin = 4;  // red button.  LOW when pressed.
int bluepin = 5;  // blue button
int greenpin = 6;   // green button
int yellowpin = 7;   // yellow button

int val;    // variable used for reading and writing data

// minimum and maximum values for movement ranges
const int rotateMax = 180;
const int shoulderMax = 180;
const int elbowMax = 180;
const int wristMax = 180;
const int clawMax = 112;
const int rotateMin = 0;
const int shoulderMin = 0;
const int elbowMin = 0;
const int wristMin = 0;
const int clawMin = 44;
const int waveMax = 30;
// const int waveMin = -30;

// parameters needed to compute arm position in r and z, given the shoulder and elbow angles
float humerusLength = 105.0;  // length in mm
float ulnaLength = 150.0;
// Offsets between control angles and real physical joint angles (angles in degrees).
// real angle = control angle - offset
// shoulder real angle is angle above horizontal
// elbow real angle is angle below straignt
// wrist real angle is angle above straight
int shoulderOffset = 5;
int elbowOffset = 5;
int wristOffset = 70; // to keep claw level, wristPos = elbowPos - shoulderPos + wristOffset

bool greenPressed = false;
bool yellowPressed = false;
bool redPressed = false;
bool bluePressed = false;
bool upPressed = false;
bool downPressed = false;
bool leftPressed = false;
bool rightPressed = false;
bool nonePressed = true;
bool leftPressedLast = false;
bool rightPressedLast = false;
// holding the yellow button opens then claw until fully open then closes until fully closed.  This variable keeps track of the direction of motion.
int clawStep = 1;
int clawOpenStep = -2;  // after up is pressed, next press of yellow button will open the claw.  This assumes you have picked up something and then want to release it after moving it.
int clawCloseStep = 1;  // close claw slowly under manual control, open it quickly
int waveStep = 2;
unsigned long currentTime = 0;
unsigned long timeStep = 20;
unsigned long endTime = 20;
unsigned long ignoreDelay = 5000;  // after left/up/blue is pressed, ignore further left/up/blue for this delay time
unsigned long ignoreEnd = 0;

// the states are:
// 0 - waiting to grab candy
// 1 - closing claw
// 2 - rotating toward the drop position
// 3 - extending arm to the drop position
// 4 - waiting to drop candy
// 5 - opening claw
// 6 - retracting arm
// 7 - rotating to pickup position
// while in states 0 or 4, button presses can control the arm manually.
// pressing up/left/blue simultaneously switches between 0-7 and 10-21 cycles.  Program jumps to 5 or 19.
// states 10 and up are for the mode where 4 different candies are in 4 specific locations, 
//  buttons select which candy to pick up
// 10 - waiting to select candy
// 11 - rotate to selected angle
// 12 - extend and go down to pickup location
// 13 - close claw to appropriate width (depends on the candy selected)
// 14 - raise candy
// 15 - retract arm
// 16 - rotate to drop angle
// 17 - extend arm
// 18 - waiting to drop candy
// 19 - opening claw
// 20 - retracting arm
// 21 - rotating to wait position. On completion goes to stste 10.
// 22 - close claw (copy of state 1).  On completion goes to state 16.
int state = 10;


int rotatePickup = 10;   // servo positions for pickup    
int shoulderPickup = 135;
int elbowPickup = 90;
// int wristPickup = 40;
int wristPickup = elbowPickup - shoulderPickup + wristOffset;
int clawClosed = clawMax;  // closed claw position

int rotateDrop = 155;   // servo positions when ready to drop
int shoulderDrop = 50;
int elbowDrop = 25;
// int wristDrop = 50;
int wristDrop = elbowDrop - shoulderDrop + wristOffset;

// these are the positions for states 10-21
// 0: waiting for pickup
// 1: green pressed (Aero)
// 2: yellow pressed (Coffee Crisp)
// 3: blue pressed (Smarties)
// 4: red pressed (Kit Kat)
// 5: arm lifted up after picking candy
int rotatePickupN[5] = {155,11,20,32,40};
int shoulderPickupN[6] = {137,90,92,95,98,71};
int elbowPickupN[6] = {126,112,117,117,118,46};
int wristPickupN[6] = {59,92,95,92,90,45};
int clawPickupN[5] = {50,95,89,91,97};
int candy = 1;
int clawOpen = clawPickupN[0];   // open claw position
// variables to store the current arm positions
int rotatePos = rotatePickup;
int shoulderPos = shoulderPickup;
int elbowPos = elbowPickup;
int wristPos = wristPickup;
int clawPos = clawOpen;


float pi180 = 3.14159265359/180.;
float pi180i = 1/pi180;
// variables to store current arm position in r and z (dimensions are mm)
float armr = humerusLength*cos(float(shoulderPos - shoulderOffset)*pi180) + ulnaLength*cos(float(shoulderPos - shoulderOffset - elbowPos + elbowOffset)*pi180);
float armz = humerusLength*sin(float(shoulderPos - shoulderOffset)*pi180) + ulnaLength*sin(float(shoulderPos - shoulderOffset - elbowPos + elbowOffset)*pi180);
float armzMin = -30;
float armzPickup = humerusLength*sin(float(shoulderPickupN[5] - shoulderOffset)*pi180) + ulnaLength*sin(float(shoulderPickupN[5] - shoulderOffset - elbowPickupN[5] + elbowOffset)*pi180);
float armd2 = (armr*armr+armz*armz);  // distance from origin to claw, squared
float armd2max = (humerusLength+ulnaLength)*(humerusLength+ulnaLength);
float armLength2 = humerusLength*humerusLength + ulnaLength*ulnaLength;  // this quantity appears often in the calculations
float armLengthd2 = humerusLength*humerusLength - ulnaLength*ulnaLength;
float twol1l2 = 2.0*humerusLength*ulnaLength;
float theta2 = 0.0;
float theta1 = 0.0;
float beta = 0.0;
float phi = 0.0;
// float rstep = 1.0;
int rotateDiff = rotatePickup - rotateDrop;
int shoulderDiff = shoulderPickup - shoulderDrop;
int elbowDiff = elbowPickup - elbowDrop;
int wristDiff = wristPickup - wristDrop;
int clawDiff = clawClosed - clawOpen;
int steps = 50;  // number of steps for each move
int j = 0;  // count the steps during moves

void setup() {
  Serial.begin(9600);
  rotate.attach(8);  // attaches the servo on pin 8 to the servo object rotate
  shoulder.attach(9);  // attaches the servo on pin 9 to the servo object shoulder
  elbow.attach(10);  // attaches the servo on pin 10 to the servo object elbow
  wrist.attach(12);  // attaches the servo on pin 12 to the servo object wrist
  claw.attach(11);  // attaches the servo on pin 11 to the servo object claw
  pinMode(redpin,INPUT_PULLUP);
  pinMode(bluepin,INPUT_PULLUP);
  pinMode(greenpin,INPUT_PULLUP);
  pinMode(yellowpin,INPUT_PULLUP);
  shoulder.write(shoulderPickupN[0]);    // set the servos to pickup positions
  elbow.write(elbowPickupN[0]); 
  wrist.write(wristPickupN[0]);
  delay(1000);                           // waits for the servos to get there
  rotate.write(rotatePickupN[0]);
  claw.write(clawPickupN[0]);
  delay(1000);
  rotatePos = rotatePickupN[0];
  shoulderPos = shoulderPickupN[0];
  elbowPos = elbowPickupN[0];
  wristPos = wristPickupN[0];
  clawPos = clawPickupN[0];
}

void loop() {
  
  currentTime = millis();
  endTime = currentTime + timeStep;
  
// check for button presses
  greenPressed = false;
  yellowPressed = false;
  redPressed = false;
  bluePressed = false;
  leftPressed = false;
  rightPressed = false;
  upPressed = false;
  downPressed = false;
  val = digitalRead(greenpin);
  if (val == LOW) {
    greenPressed = true;
    nonePressed = false;
  }
  val = digitalRead(yellowpin);
  if (val == LOW) {
    yellowPressed = true;
    nonePressed = false;
  }
  val = digitalRead(redpin);
  if (val == LOW) {
    redPressed = true;
    nonePressed = false;
  }
  val = digitalRead(bluepin);
  if (val == LOW) {
    bluePressed = true;
    nonePressed = false;
  }
  val = analogRead(hpin);
  if (val < 400) {
    rightPressed = true;
    nonePressed = false;
  }
  if (val > 800) {
    leftPressed = true;
    nonePressed = false;
  }
  val = analogRead(vpin);
  if (val < 400) {
    downPressed = true;
    nonePressed = false;
  }
  if (val > 800) {
    upPressed = true;
    nonePressed = false;
  }
  if (upPressed && leftPressed && bluePressed) {
    if (currentTime > ignoreEnd) {
      ignoreEnd = currentTime + ignoreDelay;
      if (state < 10) {
        state = 19;
      }
      else {
        state = 5;
      }
    }
  }
// When all buttons have been released, print servo positions to the serial monitor.
  if (!greenPressed && !yellowPressed && !redPressed && !bluePressed && !upPressed && !downPressed && !leftPressed && !rightPressed) {
    if (!nonePressed) {
      Serial.print("rotatePos = ");
      Serial.println(rotatePos);
      Serial.print("shoulderPos = ");
      Serial.println(shoulderPos);
      Serial.print("elbowPos = ");
      Serial.println(elbowPos);
      Serial.print("wristPos = ");
      Serial.println(wristPos);
      Serial.print("clawPos = ");
      Serial.println(clawPos);
      nonePressed = true;
    }
  }

switch (state) {
  case 0:
    if (greenPressed) state++;
    if (yellowPressed) {
      clawPos = clawPos + clawStep;
      if (clawPos > clawMax) {  // claw is closed
        clawPos = clawMax;
        clawStep = clawOpenStep;
        delay(300);
      }
      else if (clawPos < clawMin) {  // claw is open
        clawPos = clawMin;
        clawStep = clawCloseStep;
        delay(100);
      }
      else {
        claw.write(clawPos); 
      }
    }
    if (leftPressed) {
      rotatePos -= 1;
      if (rotatePos < rotateMin) rotatePos = rotateMin;
      rotate.write(rotatePos); 
    }
    if (rightPressed) {
      rotatePos += 1;
      if (rotatePos > rotateMax) rotatePos = rotateMax;
      rotate.write(rotatePos); 
    }
    if (upPressed) {
      clawStep = clawOpenStep;
      armz += 2.0;
      armd2 = (armr*armr+armz*armz);
      if (armd2 < armd2max) {
        theta2 = acos((armd2 - armLength2)/twol1l2);
        beta = atan2(armz,armr);
        phi = acos((armd2+armLengthd2)/2/humerusLength/sqrt(armd2));
        theta1 = beta + phi;
        shoulderPos = int(theta1*pi180i) + shoulderOffset;
        elbowPos = int(theta2*pi180i) + elbowOffset;
        wristPos = elbowPos - shoulderPos + wristOffset;
        if ((shoulderPos > shoulderMin)&&(shoulderPos < shoulderMax)&&(elbowPos > elbowMin)&&(elbowPos < elbowMax)&&(wristPos > wristMin)&&(wristPos < wristMax)) {
          shoulder.write(shoulderPos);
          elbow.write(elbowPos);
          wrist.write(wristPos);
        }
        else {
          armz -= 2.0;
        }
      }
      else {
        armz -= 2.0;
      }
    }
    if (downPressed) {
      armz -= 2.0;
      armd2 = (armr*armr+armz*armz);
      if (armz > armzMin) {
        theta2 = acos((armd2 - armLength2)/twol1l2);
        beta = atan2(armz,armr);
        phi = acos((armd2+armLengthd2)/2/humerusLength/sqrt(armd2));
        theta1 = beta + phi;
        shoulderPos = int(theta1*pi180i) + shoulderOffset;
        elbowPos = int(theta2*pi180i) + elbowOffset;
        wristPos = elbowPos - shoulderPos + wristOffset;
        if ((shoulderPos > shoulderMin)&&(shoulderPos < shoulderMax)&&(elbowPos > elbowMin)&&(elbowPos < elbowMax)&&(wristPos > wristMin)&&(wristPos < wristMax)) {
          shoulder.write(shoulderPos);
          elbow.write(elbowPos);
          wrist.write(wristPos);
        }
        else {
          armz += 2.0;
        }
      }
      else {
        armz += 2.0;
      }
    }
    if (redPressed&&bluePressed) {
      wristPos += waveStep;
      if (wristPos > elbowPos - shoulderPos + wristOffset + waveMax) {
        waveStep = -waveStep;
      }
      if (wristPos < elbowPos - shoulderPos + wristOffset - waveMax) {
        waveStep = -waveStep;
      }
      wrist.write(wristPos);
    }
    else if (bluePressed) {
      armr += 2;
      armd2 = (armr*armr+armz*armz);
      if (armd2 < armd2max) {
        theta2 = acos((armd2 - armLength2)/twol1l2);
        beta = atan2(armz,armr);
        phi = acos((armd2+armLengthd2)/2/humerusLength/sqrt(armd2));
        theta1 = beta + phi;
        shoulderPos = int(theta1*pi180i) + shoulderOffset;
        elbowPos = int(theta2*pi180i) + elbowOffset;
        wristPos = elbowPos - shoulderPos + wristOffset;
        if ((shoulderPos > shoulderMin)&&(shoulderPos < shoulderMax)&&(elbowPos > elbowMin)&&(elbowPos < elbowMax)&&(wristPos > wristMin)&&(wristPos < wristMax)) {
          shoulder.write(shoulderPos);
          elbow.write(elbowPos);
          wrist.write(wristPos);
        }
        else {
          armr -= 2.0;
        }
      }
      else {
        armr -= 2.0;
      }
    }
    else if (redPressed) {
      armr -= 2.0;
      armd2 = (armr*armr+armz*armz);
      if (armz > 0.0) {
        theta2 = acos((armd2 - armLength2)/twol1l2);
        beta = atan2(armz,armr);
        phi = acos((armd2+armLengthd2)/2/humerusLength/sqrt(armd2));
        theta1 = beta + phi;
        shoulderPos = int(theta1*pi180i) + shoulderOffset;
        elbowPos = int(theta2*pi180i) + elbowOffset;
        wristPos = elbowPos - shoulderPos + wristOffset;
        if ((shoulderPos > shoulderMin)&&(shoulderPos < shoulderMax)&&(elbowPos > elbowMin)&&(elbowPos < elbowMax)&&(wristPos > wristMin)&&(wristPos < wristMax)) {
          shoulder.write(shoulderPos);
          elbow.write(elbowPos);
          wrist.write(wristPos);
        }
        else {
          armr += 2.0;
        }
      }
      else {
        armr += 2.0;
      }
    }
    break;
  case 1:
    // close claw.  
    // If clawStep is indicating the claw was last closed by user, skip closing the claw.
    if (clawStep != clawOpenStep) {
      state++;
      break;
    }
    clawPos += 2;
    if (clawPos >= clawClosed) {
      clawPos = clawClosed;
      state++;
      delay(500);
    }
    claw.write(clawPos);
    break;
  case 2:
    // rotate to drop position
    rotatePos += 2;
    if (rotatePos >= rotateDrop) {
      rotatePos = rotateDrop;
      state++;
    }
    rotate.write(rotatePos);
    break;
  case 3:
    // extend arm
    shoulderPos -= 2;
    elbowPos -= 2;
    if (elbowPos <= elbowDrop) elbowPos = elbowDrop;
    if (shoulderPos <= shoulderDrop) {
      shoulderPos = shoulderDrop;
      armr = humerusLength*cos(float(shoulderPos - shoulderOffset)*pi180) + ulnaLength*cos(float(shoulderPos - shoulderOffset - elbowPos + elbowOffset)*pi180);
      armz = humerusLength*sin(float(shoulderPos - shoulderOffset)*pi180) + ulnaLength*sin(float(shoulderPos - shoulderOffset - elbowPos + elbowOffset)*pi180);
      state++;
    }
    wristPos = elbowPos - shoulderPos + wristOffset;
    shoulder.write(shoulderPos);
    elbow.write(elbowPos);
    wrist.write(wristPos);
    break;
  case 4:
    if (greenPressed) {
      state++;
      clawStep = clawOpenStep;
    }
    if (yellowPressed) {
      clawPos = clawPos + clawStep;
      if (clawPos > clawMax) {  // claw is closed
        clawPos = clawMax;
        clawStep = clawOpenStep;
        delay(300);
      }
      else if (clawPos < clawMin) {  // claw is open
        clawPos = clawMin;
        clawStep = clawCloseStep;
        delay(100);
      }
      else {
        claw.write(clawPos); 
      }
    }
    if (leftPressed) {
      rotatePos -= 1;
      if (rotatePos < rotateMin) rotatePos = rotateMin;
      rotate.write(rotatePos); 
    }
    if (rightPressed) {
      rotatePos += 1;
      if (rotatePos > rotateMax) rotatePos = rotateMax;
      rotate.write(rotatePos); 
    }
    if (upPressed) {
      clawStep = clawOpenStep;
      armz += 2.0;
      armd2 = (armr*armr+armz*armz);
      if (armd2 < armd2max) {
        theta2 = acos((armd2 - armLength2)/twol1l2);
        beta = atan2(armz,armr);
        phi = acos((armd2+armLengthd2)/2/humerusLength/sqrt(armd2));
        theta1 = beta + phi;
        shoulderPos = int(theta1*pi180i) + shoulderOffset;
        elbowPos = int(theta2*pi180i) + elbowOffset;
        wristPos = elbowPos - shoulderPos + wristOffset;
        if ((shoulderPos > shoulderMin)&&(shoulderPos < shoulderMax)&&(elbowPos > elbowMin)&&(elbowPos < elbowMax)&&(wristPos > wristMin)&&(wristPos < wristMax)) {
          shoulder.write(shoulderPos);
          elbow.write(elbowPos);
          wrist.write(wristPos);
        }
        else {
          armz -= 2.0;
        }
      }
      else {
        armz -= 2.0;
      }
    }
    if (downPressed) {
      armz -= 2.0;
      armd2 = (armr*armr+armz*armz);
      if (armz > armzMin) {
        theta2 = acos((armd2 - armLength2)/twol1l2);
        beta = atan2(armz,armr);
        phi = acos((armd2+armLengthd2)/2/humerusLength/sqrt(armd2));
        theta1 = beta + phi;
        shoulderPos = int(theta1*pi180i) + shoulderOffset;
        elbowPos = int(theta2*pi180i) + elbowOffset;
        wristPos = elbowPos - shoulderPos + wristOffset;
        if ((shoulderPos > shoulderMin)&&(shoulderPos < shoulderMax)&&(elbowPos > elbowMin)&&(elbowPos < elbowMax)&&(wristPos > wristMin)&&(wristPos < wristMax)) {
          shoulder.write(shoulderPos);
          elbow.write(elbowPos);
          wrist.write(wristPos);
        }
        else {
          armz += 2.0;
        }
      }
      else {
        armz += 2.0;
      }
    }
    if (redPressed&&bluePressed) {
      wristPos += waveStep;
      if (wristPos > elbowPos - shoulderPos + wristOffset + waveMax) {
        waveStep = -waveStep;
      }
      if (wristPos < elbowPos - shoulderPos + wristOffset - waveMax) {
        waveStep = -waveStep;
      }
      wrist.write(wristPos);
    }
    else if (bluePressed) {
      armr += 2.0;
      armd2 = (armr*armr+armz*armz);
      if (armd2 < armd2max) {
        theta2 = acos((armd2 - armLength2)/twol1l2);
        beta = atan2(armz,armr);
        phi = acos((armd2+armLengthd2)/2/humerusLength/sqrt(armd2));
        theta1 = beta + phi;
        shoulderPos = int(theta1*pi180i) + shoulderOffset;
        elbowPos = int(theta2*pi180i) + elbowOffset;
        wristPos = elbowPos - shoulderPos + wristOffset;
        if ((shoulderPos > shoulderMin)&&(shoulderPos < shoulderMax)&&(elbowPos > elbowMin)&&(elbowPos < elbowMax)&&(wristPos > wristMin)&&(wristPos < wristMax)) {
          shoulder.write(shoulderPos);
          elbow.write(elbowPos);
          wrist.write(wristPos);
        }
        else {
          armr -= 2.0;
        }
      }
      else {
        armr -= 2.0;
      }
    }
    else if (redPressed) {
      armr -= 2.0;
      armd2 = (armr*armr+armz*armz);
      if (armz > 0.0) {
        theta2 = acos((armd2 - armLength2)/twol1l2);
        beta = atan2(armz,armr);
        phi = acos((armd2+armLengthd2)/2/humerusLength/sqrt(armd2));
        theta1 = beta + phi;
        shoulderPos = int(theta1*pi180i) + shoulderOffset;
        elbowPos = int(theta2*pi180i) + elbowOffset;
        wristPos = elbowPos - shoulderPos + wristOffset;
        if ((shoulderPos > shoulderMin)&&(shoulderPos < shoulderMax)&&(elbowPos > elbowMin)&&(elbowPos < elbowMax)&&(wristPos > wristMin)&&(wristPos < wristMax)) {
          shoulder.write(shoulderPos);
          elbow.write(elbowPos);
          wrist.write(wristPos);
        }
        else {
          armr += 2.0;
        }
      }
      else {
        armr += 2.0;
      }
    }
    break;
  case 5:
    // open claw
    clawPos -= 2;
    if (clawPos < clawOpen) {
      clawPos = clawOpen;
      state++;
      delay(50);
    }
    claw.write(clawPos);
    break;
  case 6:
    // retract arm
    shoulderPos += 2;
    elbowPos += 2;
    if (elbowPos >= elbowPickup) elbowPos = elbowPickup;
    if (shoulderPos >= shoulderPickup) {
      shoulderPos = shoulderPickup;
      state++;
    }
    wristPos = elbowPos - shoulderPos + wristOffset;
    shoulder.write(shoulderPos);
    elbow.write(elbowPos);
    wrist.write(wristPos);
    break;
  case 7:
    // rotate to pickup position
    rotatePos -= 2;
    if (rotatePos <= rotatePickup) {
      rotatePos = rotatePickup;
      armr = humerusLength*cos(float(shoulderPos - shoulderOffset)*pi180) + ulnaLength*cos(float(shoulderPos - shoulderOffset - elbowPos + elbowOffset)*pi180);
      armz = humerusLength*sin(float(shoulderPos - shoulderOffset)*pi180) + ulnaLength*sin(float(shoulderPos - shoulderOffset - elbowPos + elbowOffset)*pi180);
      state = 0;
    }
    rotate.write(rotatePos);
    break;
  case 10:
    // wait for candy selection
    if (greenPressed) {
      candy = 1;
      state++;
    }
    if (yellowPressed) {
      candy = 2;
      state++;
    }
    if (bluePressed) {
      candy = 3;
      state++;
    }
    if (redPressed) {
      candy = 4;
      state++;
    }
    if (leftPressed) {
      if (!leftPressedLast) {
        leftPressedLast = true;
        rotatePos -= 1;
        rotate.write(rotatePos); 
        rotatePickupN[0] -= 1;
        rotatePickupN[1] -= 1;
        rotatePickupN[2] -= 1;
        rotatePickupN[3] -= 1;
        rotatePickupN[4] -= 1;
      }
    }
    else {
      leftPressedLast = false;
    }
    if (rightPressed) {
      if (!rightPressedLast) {
        rightPressedLast = true;
        rotatePos += 1;
        rotate.write(rotatePos); 
        rotatePickupN[0] += 1;
        rotatePickupN[1] += 1;
        rotatePickupN[2] += 1;
        rotatePickupN[3] += 1;
        rotatePickupN[4] += 1;
      }
    }
    else {
      rightPressedLast = false;
    }
    if (downPressed) {
      state = 22;
    }
    break;
  case 11: 
    // rotate to pick up candy
    if (candy<5) {
      rotatePos -= 2;
      if (rotatePos <= rotatePickupN[candy]) {
        rotatePos = rotatePickupN[candy];
        delay(200);
        state++;
      }
    }
    else {
      rotatePos += 2;
      if (rotatePos >= rotatePickupN[candy]) {
        rotatePos = rotatePickupN[candy];
        delay(200);
        state++;
      }
    }
    rotate.write(rotatePos);
    break;
  case 12:
    // extend to pick up candy
    shoulderPos -= 2;
    elbowPos -= 2;
    if (elbowPos <= elbowPickupN[candy]) elbowPos = elbowPickupN[candy];
    if (shoulderPos <= shoulderPickupN[candy]) {
      shoulderPos = shoulderPickupN[candy];
      armr = humerusLength*cos(float(shoulderPos - shoulderOffset)*pi180) + ulnaLength*cos(float(shoulderPos - shoulderOffset - elbowPos + elbowOffset)*pi180);
      armz = humerusLength*sin(float(shoulderPos - shoulderOffset)*pi180) + ulnaLength*sin(float(shoulderPos - shoulderOffset - elbowPos + elbowOffset)*pi180);
      state++;
    }
    wristPos = elbowPos - shoulderPos + wristOffset;
    shoulder.write(shoulderPos);
    elbow.write(elbowPos);
    wrist.write(wristPos);
    break;
  case 13:
    // close claw.  
    clawPos += 2;
    if (clawPos >= clawPickupN[candy]) {
      clawPos = clawPickupN[candy];
      state++;
    }
    claw.write(clawPos);
    break;
  case 14:
    // lift candy
      armz += 2.0;
      armd2 = (armr*armr+armz*armz);
      if (armz < armzPickup) {
        theta2 = acos((armd2 - armLength2)/twol1l2);
        beta = atan2(armz,armr);
        phi = acos((armd2+armLengthd2)/2/humerusLength/sqrt(armd2));
        theta1 = beta + phi;
        shoulderPos = int(theta1*pi180i) + shoulderOffset;
        elbowPos = int(theta2*pi180i) + elbowOffset;
        wristPos = elbowPos - shoulderPos + wristOffset;
        shoulder.write(shoulderPos);
        elbow.write(elbowPos);
        wrist.write(wristPos);
      }
      else {
        state++;
      }
     break;
  case 15:
    // retract arm
    shoulderPos += 2;
    elbowPos += 2;
    if (elbowPos >= elbowPickupN[0]) elbowPos = elbowPickupN[0];
    if (shoulderPos >= shoulderPickupN[0]) {
      shoulderPos = shoulderPickupN[0];
      state++;
    }
    wristPos = elbowPos - shoulderPos + wristOffset;
    shoulder.write(shoulderPos);
    elbow.write(elbowPos);
    wrist.write(wristPos);
    break;
  case 16:
    // rotate to drop position
    rotatePos += 2;
    if (rotatePos >= rotateDrop) {
      rotatePos = rotateDrop;
      state++;
    }
    rotate.write(rotatePos);
    break;
  case 17:
    // extend arm
    shoulderPos -= 2;
    elbowPos -= 2;
    if (elbowPos <= elbowDrop) elbowPos = elbowDrop;
    if (shoulderPos <= shoulderDrop) {
      shoulderPos = shoulderDrop;
      armr = humerusLength*cos(float(shoulderPos - shoulderOffset)*pi180) + ulnaLength*cos(float(shoulderPos - shoulderOffset - elbowPos + elbowOffset)*pi180);
      armz = humerusLength*sin(float(shoulderPos - shoulderOffset)*pi180) + ulnaLength*sin(float(shoulderPos - shoulderOffset - elbowPos + elbowOffset)*pi180);
      state++;
    }
    wristPos = elbowPos - shoulderPos + wristOffset;
    shoulder.write(shoulderPos);
    elbow.write(elbowPos);
    wrist.write(wristPos);
    break;
  case 18:
    if (downPressed) {
      state++;
      clawStep = clawOpenStep;
    }
    break;
  case 19:
    // open claw
    clawPos -= 2;
    if (clawPos < clawPickupN[0]) {
      clawPos = clawPickupN[0];
      state++;
      delay(50);
    }
    claw.write(clawPos);
    break;
  case 20:
    // retract arm
    // first check whether shoulderPos > shoulderPickupN[0], which could happen after up/left/blue press.
    if (shoulderPos > shoulderPickupN[0]) {
      shoulderPos -= 2;
      elbowPos -= 2;
    }
    else {
      shoulderPos += 2;
      elbowPos += 2;
      if (elbowPos >= elbowPickupN[0]) elbowPos = elbowPickupN[0];
      if (shoulderPos >= shoulderPickupN[0]) {
        shoulderPos = shoulderPickupN[0];
        state++;
      }
      wristPos = elbowPos - shoulderPos + wristOffset;
    }
    shoulder.write(shoulderPos);
    elbow.write(elbowPos);
    wrist.write(wristPos);
    break;
  case 21:
    // rotate to pickup position
    rotatePos -= 2;
    if (rotatePos <= rotatePickupN[0]) {
      rotatePos = rotatePickupN[0];
      armr = humerusLength*cos(float(shoulderPos - shoulderOffset)*pi180) + ulnaLength*cos(float(shoulderPos - shoulderOffset - elbowPos + elbowOffset)*pi180);
      armz = humerusLength*sin(float(shoulderPos - shoulderOffset)*pi180) + ulnaLength*sin(float(shoulderPos - shoulderOffset - elbowPos + elbowOffset)*pi180);
      state = 10;
    }
    rotate.write(rotatePos);
    break;
  case 22:
    // close claw.  
    clawPos += 2;
    if (clawPos >= clawClosed) {
      clawPos = clawClosed;
      state = 16;
      delay(500);
    }
    claw.write(clawPos);
    break;
  default:
    // should never get here
    break;

}

  do {
   currentTime = millis();      // wait for step time to be elapsed.  Doing it this way the calculation time is included in the step time.
  } while (currentTime<endTime);

 // delay(20);
  
}
